package api

import (
	"errors"
	"spu3d/db/model"
	"spu3d/db/repo"
	"spu3d/log"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func BoxLib(r *GinRouter) {
	r.POST("/boxlib/create", CreateBoxLib)
	r.POST("/boxlib/delete/:id", DeleteBoxLib)
	r.GET("/boxlib/list", BoxLibList)
	r.GET("/boxlib/detail/:id", BoxLibDetail)
	r.POST("/boxlib/update", UpdateBoxLib)
}

func CreateBoxLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.BoxLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	box.CreateTime = time.Now()
	box.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionBoxLib, &box)
}

func DeleteBoxLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionBoxLib, _id)
}

func BoxLibList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	catsConf, err := GetSpu3dBoxLibCategory(apictx)
	if err != nil {
		return nil, err
	}
	err = ParseCategories(query, apictx.CreateRepoCtx(), catsConf)
	if err != nil {
		return nil, err
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionBoxLib,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.M{"createTime": -1},
	})
}

func BoxLibDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	box := &model.BoxLib{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionBoxLib,
		Query:       repo.Map{"_id": id},
	}, box)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return box, nil
}

func UpdateBoxLib(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.BoxLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if box.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionBoxLib, box.Id.Hex(), &box)
}
