package api

import (
	"errors"
	"spu3d/db/model"
	"spu3d/db/repo"
	"spu3d/log"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Gallery(r *GinRouter) {
	r.POSTJWT("/gallery/create", CreateGallery)
	r.POSTJWT("/gallery/delete/:id", DeleteGallery)
	r.GETJWT("/gallery/list", GalleryList)
	r.GETJWT("/gallery/detail/:id", GalleryDetail)
	r.POSTJWT("/gallery/update", UpdateGallery)
}

func CreateGallery(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var gallery model.Gallery
	err := c.ShouldBindJSON(&gallery)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	gallery.CreateTime = time.Now()
	gallery.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionGallery, &gallery)
}

func DeleteGallery(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionGallery, _id)
}

func GalleryList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionGallery,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.M{"createTime": -1},
	})
}

func GalleryDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	gallery := &model.Gallery{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionGallery,
		Query:       repo.Map{"_id": id},
	}, gallery)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return gallery, nil
}

func UpdateGallery(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var gallery model.Gallery
	err := c.ShouldBindJSON(&gallery)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if gallery.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionGallery, gallery.Id.Hex(), &gallery)
}
